VERSION 5.00
Begin VB.UserControl Gauge 
   Appearance      =   0  'Flat
   BackColor       =   &H80000005&
   ClientHeight    =   270
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   1890
   ScaleHeight     =   18
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   126
   ToolboxBitmap   =   "Gauge.ctx":0000
   Begin VB.PictureBox Picture2D 
      Appearance      =   0  'Flat
      BackColor       =   &H80000005&
      ForeColor       =   &H80000008&
      Height          =   255
      Left            =   720
      ScaleHeight     =   15
      ScaleMode       =   3  'Pixel
      ScaleWidth      =   23
      TabIndex        =   1
      Top             =   0
      Width           =   375
   End
   Begin VB.PictureBox Picture3D 
      Height          =   255
      Left            =   120
      ScaleHeight     =   13
      ScaleMode       =   3  'Pixel
      ScaleWidth      =   21
      TabIndex        =   0
      Top             =   0
      Width           =   375
   End
End
Attribute VB_Name = "Gauge"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' *********************************************
' A bar gauge.
' *********************************************
Option Explicit

' Pi is used in circular calculations.
Private Const PI = 3.14159265

' The gauge's value.
Private the_value As Long

' The Min and Max values.
Private the_min As Long
Private the_max As Long

' Arrow key change amount.
Private value_change As Long

' Horizontal and vertical orientations.
Public Enum GaugeOrientation
    orientLeftToRight
    orientRightToLeft
    orientBottomToTop
    orientTopToBottom
End Enum
Private the_orientation As GaugeOrientation

' Gauge styles.
Public Enum GaugeStyle
    styleBar
    styleDial
    stylePic
    styleTic
    styleWid
End Enum
Private the_style As GaugeStyle

' Determines 2D or 3D appearance.
Public Enum AppearanceValue
    appears2D
    appears3D
End Enum
Private the_appearance As AppearanceValue

' The spacing between width tic marks.
Private tic_spacing As Integer

' The picture to display in the covered area.
Private fill_picture As StdPicture

' The name of the picture URL.
Private fill_picture_url As String

' The width of the tics and gaps.
Private tic_width As Integer

' The color for the narrow area.
Private narrow_color As OLE_COLOR

' Thickness of narrow part as a percentage of
' the thick part.
Private narrow_percent As Integer

' The currently visible picture box.
Dim the_canvas As PictureBox

' Default property values.
Private Const dflt_BackColor = &H8000000F   ' Light gray.
Private Const dflt_ForeColor = vbBlack
Private Const dflt_FillColor = &H8000000F   ' Light gray.
Private Const dflt_Max = 100
Private Const dflt_Min = 0
Private Const dflt_Orientation = orientLeftToRight
Private Const dflt_Style = styleBar
Private Const dflt_Value = 50
Private Const dflt_KeyChangeAmount = 1
Private Const dflt_Appearance = appears3D
Private Const dflt_Enabled = True
Private Const dflt_TicSpacing = 10
Private Const dflt_TicWidth = 4
Private Const dflt_NarrowColor = &H8000000C ' Dark gray.
Private Const dflt_NarrowPercent = 50
Private Const dflt_FillPictureURL = ""

' Indicates the user is selecting a value.
Private dragging As Boolean

' Indicates the control has the focus.
Private has_focus As Boolean

' We're in the middle of loading.
Private loading As Boolean

' *******************
' Event declarations.
' *******************
' The value is changing (MouseDown, MouseMove).
Public Event Scroll()
Attribute Scroll.VB_Description = "This event triggers as the user scrolls the value"

' The value has changed (MouseUp).
Public Event Change()
Attribute Change.VB_Description = "This event triggers when the control's value changes"
' ************************************************
' Return the arc tangent of Y/X taking into
' account the proper quadrant.
' ************************************************
Public Function Arctan2(X As Single, Y As Single) As Single
Const PI = 3.14159265
Const PI_OVER_2 = PI / 2

Dim theta As Single

    If X = 0 Then
        If Y > 0 Then
            Arctan2 = PI_OVER_2
        Else
            Arctan2 = -PI_OVER_2
        End If
    Else
        theta = Atn(Y / X)
        If X < 0 Then theta = PI + theta
        Arctan2 = theta
    End If
End Function

' *********************************************
' Get the narrow color.
' *********************************************
Public Property Get NarrowColor() As OLE_COLOR
Attribute NarrowColor.VB_Description = "Color of narrow bar for styleWid"
    NarrowColor = narrow_color
End Property
' *********************************************
' Set the color to fill the narrow area with.
' *********************************************
Public Property Let NarrowColor(clr As OLE_COLOR)
    narrow_color = clr

    PropertyChanged "NarrowColor"   ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Thickness of narrow part as a percentage of
' the thick part.
' *********************************************
Public Property Get NarrowPercent() As Integer
Attribute NarrowPercent.VB_Description = "Width of narrow bar as percent of thick bar for styleWid"
    NarrowPercent = narrow_percent
End Property
' *********************************************
' Set NarrowPercent.
' *********************************************
Public Property Let NarrowPercent(val As Integer)
    narrow_percent = val

    PropertyChanged "NarrowPercent" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Paint_Wid()
Dim v As Single
Dim v1 As Single
Dim v2 As Single
Dim wid As Single
Dim hgt As Single

    ' Do not redraw while loading.
    If loading Then Exit Sub
    
    ' Start from scratch.
    the_canvas.Cls
                        
    ' Draw the narrow part.
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    Select Case the_orientation
        Case orientLeftToRight
            v1 = (hgt * NarrowPercent) / 200#
            v2 = hgt - v1
            the_canvas.Line (0, v1)-(wid, v2), NarrowColor, BF
        Case orientRightToLeft
            v1 = (hgt * NarrowPercent) / 200#
            v2 = hgt - v1
            the_canvas.Line (0, v1)-(wid, v2), NarrowColor, BF
        Case orientBottomToTop
            v1 = (wid * NarrowPercent) / 200#
            v2 = wid - v1
            the_canvas.Line (v1, 0)-(v2, hgt), NarrowColor, BF
        Case orientTopToBottom
            v1 = (wid * NarrowPercent) / 200#
            v2 = wid - v1
            the_canvas.Line (v1, 0)-(v2, hgt), NarrowColor, BF
    End Select

    ' See how much of the gauge should be filled.
    If the_min >= the_max Then
        ' Fill in nothing.
        v = 0#
    Else
        v = (the_value - the_min) / (the_max - the_min)
    End If
    
    ' Convert the value to pixels.
    If the_orientation = orientLeftToRight Or _
       the_orientation = orientRightToLeft _
    Then
        v = v * wid
    Else
        v = v * hgt
    End If

    ' Draw the gauge with the correct orientation.
    Select Case the_orientation
        Case orientLeftToRight
            the_canvas.Line (0, 0)-(v, hgt), , BF
        Case orientRightToLeft
            the_canvas.Line (wid, 0)-(wid - v, hgt), , BF
        Case orientBottomToTop
            the_canvas.Line (0, hgt)-(wid, hgt - v), , BF
        Case orientTopToBottom
            the_canvas.Line (0, 0)-(wid, v), , BF
    End Select

    ' If the gauge has the focus, outline it.
    If has_focus Then
        the_canvas.DrawStyle = vbDot
        the_canvas.DrawMode = vbInvert
        the_canvas.Line (0, 0)- _
            (wid - 1, hgt - 1), _
            , B
        the_canvas.DrawStyle = vbSolid
        the_canvas.DrawMode = vbCopyPen
    End If
End Sub

' *********************************************
' Get the stripe width.
' *********************************************
Public Property Get TicWidth() As Long
Attribute TicWidth.VB_Description = "Size of tic marks in pixels for styleTic"
    TicWidth = tic_width
End Property
' *********************************************
' Set the stripe width.
' *********************************************
Public Property Let TicWidth(val As Long)
    If val < 1 Then val = 1
    tic_width = val

    PropertyChanged "TicWidth" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Paint_Tic()
Dim v As Single
Dim x1 As Single
Dim x2 As Single
Dim dx As Single
Dim wid As Single
Dim hgt As Single

    ' Do not redraw while loading.
    If loading Then Exit Sub
    
    ' Start from scratch.
    the_canvas.Cls
                        
    ' See how much of the gauge should be filled.
    If the_min >= the_max Then
        ' Fill in nothing.
        v = 0#
    Else
        v = (the_value - the_min) / (the_max - the_min)
    End If
        
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    
    ' Convert the value to pixels.
    If the_orientation = orientLeftToRight Or _
       the_orientation = orientRightToLeft _
    Then
        v = v * wid
    Else
        v = v * hgt
    End If

    ' Draw the gauge with the correct orientation.
    dx = TicWidth
    x1 = 0
    Do While x1 < v
        x2 = x1 + dx
        If x2 > v Then x2 = v
        
        Select Case the_orientation
            Case orientLeftToRight
                the_canvas.Line (x1, 0)-(x2, hgt), , BF
            Case orientRightToLeft
                the_canvas.Line (wid - x1, 0)-(wid - x2, hgt), , BF
        
            Case orientBottomToTop
                the_canvas.Line (0, hgt - x1)-(wid, hgt - x2), , BF
        
            Case orientTopToBottom
                the_canvas.Line (0, x1)-(wid, x2), , BF
        
        End Select
        
        x1 = x2 + dx + 2
    Loop

    ' If the gauge has the focus, outline it.
    If has_focus Then
        the_canvas.DrawStyle = vbDot
        the_canvas.DrawMode = vbInvert
        the_canvas.Line (0, 0)- _
            (wid - 1, hgt - 1), _
            , B
        the_canvas.DrawStyle = vbSolid
        the_canvas.DrawMode = vbCopyPen
    End If
End Sub

' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Paint_Dial()
Dim r1 As Single    ' Outer radius.
Dim r2 As Single    ' Inner (tic) radius.
Dim x0 As Single    ' The origin of the circle.
Dim y0 As Single
Dim x1 As Single
Dim y1 As Single
Dim x2 As Single
Dim y2 As Single
Dim wid As Single
Dim hgt As Single
Dim theta As Single
Dim dtheta As Single
Dim stheta As Single
Dim ctheta As Single

    ' Do not redraw while loading.
    If loading Then Exit Sub
    
    ' Start from scratch.
    the_canvas.Cls
                        
    ' Don't bother if Min >= Max.
    If the_min >= the_max Then Exit Sub
                        
    'Draw the circle.
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    r1 = hgt - 2
    r2 = wid / 2 - 1
    If r2 < r1 Then r1 = r2
    r2 = 0.9 * r1
    x0 = wid / 2
    y0 = hgt - 1
    the_canvas.FillStyle = vbSolid
    the_canvas.Circle (x0, y0), r1, , -0.0000001, -PI
    the_canvas.FillStyle = vbFSTransparent
    
    ' Draw the tic marks.
    If tic_spacing > 0 Then
        dtheta = PI * tic_spacing / (the_max - the_min)
        For theta = dtheta To PI Step dtheta
            If Orientation = orientLeftToRight Then
                ctheta = Cos(PI - theta)
            Else
                ctheta = Cos(theta)
            End If
            stheta = Sin(theta)
            x1 = x0 + r1 * ctheta
            x2 = x0 + r2 * ctheta
            y1 = y0 - r1 * stheta
            y2 = y0 - r2 * stheta
            
            the_canvas.Line (x1, y1)-(x2, y2)
        Next theta
    End If
    
    ' See where the needle belongs.
    If the_orientation = orientLeftToRight Then
        theta = PI * (1 - (the_value - the_min) / (the_max - the_min))
    Else
        theta = PI * (the_value - the_min) / (the_max - the_min)
    End If
    
    ' Draw the needle.
    the_canvas.Line (x0, y0)-(x0 + r1 * Cos(theta), y0 - r1 * Sin(theta))

    ' If the gauge has the focus, outline it.
    If has_focus Then
        the_canvas.DrawStyle = vbDot
        the_canvas.DrawMode = vbInvert
        the_canvas.Line (0, 0)- _
            (wid - 1, hgt - 1), _
            , B
        the_canvas.DrawStyle = vbSolid
        the_canvas.DrawMode = vbCopyPen
    End If
End Sub

' *********************************************
' Get the picture to fill with.
' *********************************************
Public Property Get FillPicture() As StdPicture
Attribute FillPicture.VB_Description = "Picture used by stylePic"
    Set FillPicture = fill_picture
End Property
' *********************************************
' Get the URL of the picture to fill with.
' *********************************************
Public Property Get FillPictureURL() As String
    FillPictureURL = fill_picture_url
End Property

' *********************************************
' Set the picture to fill with.
' *********************************************
Public Property Set FillPicture(pic As StdPicture)
    Set fill_picture = pic

    PropertyChanged "FillPicture" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Load the fill picture from a URL.
' *********************************************
Public Property Let FillPictureURL(url As String)
    fill_picture_url = url

    ' If it's run time start the read.
    If Ambient.UserMode And url <> "" Then
        AsyncRead url, vbAsyncTypePicture, _
            "FillPictureURL"
    End If
    
    PropertyChanged "FillPictureURL" ' Tell VB.
    
    ' The gauge is repainted when the read
    ' finshes.
End Property

' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Paint_Pic()
Dim v As Single
Dim wid As Single
Dim hgt As Single
Dim x1 As Integer
Dim y1 As Integer
Dim x2 As Integer
Dim y2 As Integer
Dim pic_wid As Integer
Dim pic_hgt As Integer

    ' Do not redraw while loading.
    If loading Then Exit Sub
    
    ' If there is no picture, use Bar style.
    If FillPicture Is Nothing Then
        Paint_Bar
        Exit Sub
    End If
    
    ' Start from scratch.
    the_canvas.Cls
                        
    ' See how much of the gauge should be filled.
    If the_min >= the_max Then
        ' Fill in nothing.
        v = 0#
    Else
        v = (the_value - the_min) / (the_max - the_min)
    End If
        
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    
    ' Convert the value to pixels.
    If the_orientation = orientLeftToRight Or _
       the_orientation = orientRightToLeft _
    Then
        v = v * wid
    Else
        v = v * hgt
    End If
    
    ' See how big the picture is.
    pic_wid = CInt(ScaleX(FillPicture.Width, vbHimetric, vbPixels))
    pic_hgt = CInt(ScaleY(FillPicture.Height, vbHimetric, vbPixels))
    
    ' Draw the gauge with the correct orientation.
    Select Case the_orientation
        Case orientLeftToRight
            x1 = 0
            y1 = 0
            Do While x1 < v
                If x1 + pic_wid > v Then pic_wid = v - x1
                If pic_wid > 0 Then
                    the_canvas.PaintPicture FillPicture, _
                        x1, y1, pic_wid, pic_hgt, _
                        , , pic_wid, pic_hgt
                    x1 = x1 + pic_wid
                Else
                    x1 = v + 1
                End If
            Loop
            
        Case orientRightToLeft
            x1 = pic_wid
            y1 = 0
            Do While x1 - pic_wid < v
                If x1 <= v Then
                    the_canvas.PaintPicture FillPicture, _
                        wid - x1, y1, pic_wid, pic_hgt, _
                        , , pic_wid, pic_hgt
                    x1 = x1 + pic_wid
                Else
                    x2 = x1 - v
                    x1 = v
                    pic_wid = pic_wid - x2
                    If pic_wid > 0 Then
                        the_canvas.PaintPicture FillPicture, _
                            wid - x1, y1, pic_wid, pic_hgt, _
                            x2, , pic_wid, pic_hgt
                    End If
                    Exit Do
                End If
            Loop

        Case orientBottomToTop
            x1 = 0
            y1 = pic_hgt
            Do While y1 - pic_hgt < v
                If y1 <= v Then
                    the_canvas.PaintPicture FillPicture, _
                        x1, hgt - y1, pic_wid, pic_hgt, _
                        , , pic_wid, pic_hgt
                    y1 = y1 + pic_wid
                Else
                    y2 = y1 - v
                    y1 = v
                    pic_hgt = pic_hgt - y2
                    If pic_hgt > 0 Then
                        the_canvas.PaintPicture FillPicture, _
                            x1, hgt - y1, pic_wid, pic_hgt, _
                            , y2, pic_wid, pic_hgt
                    End If
                    Exit Do
                End If
            Loop
        
        Case orientTopToBottom
            x1 = 0
            y1 = 0
            Do While y1 < v
                If y1 + pic_hgt > v Then pic_hgt = v - y1
                If pic_hgt > 0 Then
                    the_canvas.PaintPicture FillPicture, _
                        x1, y1, pic_wid, pic_hgt, _
                        , , pic_wid, pic_hgt
                    y1 = y1 + pic_wid
                Else
                    y1 = v + 1
                End If
            Loop
    
    End Select
    
    ' If the gauge has the focus, outline it.
    If has_focus Then
        the_canvas.DrawStyle = vbDot
        the_canvas.DrawMode = vbInvert
        the_canvas.Line (0, 0)- _
            (wid - 1, hgt - 1), _
            , B
        the_canvas.DrawStyle = vbSolid
        the_canvas.DrawMode = vbCopyPen
    End If
End Sub

' *********************************************
' Get the background color.
' *********************************************
Public Property Get BackColor() As OLE_COLOR
Attribute BackColor.VB_Description = "For styles other than Dial, fills the gauge when Value = Min"
Attribute BackColor.VB_UserMemId = -501
    BackColor = UserControl.BackColor
End Property
' *********************************************
' Set the background color.
' *********************************************
Public Property Let BackColor(clr As OLE_COLOR)
    UserControl.BackColor = clr
    Picture2D.BackColor = clr
    Picture3D.BackColor = clr

    PropertyChanged "BackColor" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Set the color to fill with.
' *********************************************
Public Property Let ForeColor(clr As OLE_COLOR)
    UserControl.ForeColor = clr
    Picture2D.ForeColor = clr
    Picture3D.ForeColor = clr

    PropertyChanged "ForeColor" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Get the color to fill with.
' *********************************************
Public Property Get ForeColor() As OLE_COLOR
Attribute ForeColor.VB_Description = "For styles other than styleDial, fills the gauge when Value = Max"
Attribute ForeColor.VB_UserMemId = -513
    ForeColor = UserControl.ForeColor
End Property
' *********************************************
' Set the minimum value.
' *********************************************
Public Property Let Min(val As Long)
Attribute Min.VB_Description = "Minimum allowed Value"
    the_min = val

    PropertyChanged "Min"   ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Get the minimum value.
' *********************************************
Public Property Get Min() As Long
    Min = the_min
End Property
' *********************************************
' Return the value of Enabled.
' *********************************************
Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Allow user interaction"
Attribute Enabled.VB_UserMemId = -514
    Enabled = UserControl.Enabled
End Property
' *********************************************
' Set the maximum value.
' *********************************************
Public Property Let Max(val As Long)
Attribute Max.VB_Description = "Maximum allowed Value"
    the_max = val

    PropertyChanged "Max"   ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Set the Appearance.
' *********************************************
Public Property Let Appearance(val As AppearanceValue)
    the_appearance = val
    
    ' Make the correct picture box visible.
    If the_appearance = appears2D Then
        Picture2D.Visible = True
        Picture3D.Visible = False
        Set the_canvas = Picture2D
    Else
        Picture2D.Visible = False
        Picture3D.Visible = True
        Set the_canvas = Picture3D
    End If
    
    PropertyChanged "Appearance"    ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property

' *********************************************
' Set the amount to change the value when the
' user presses an arrow key.
' *********************************************
Public Property Let KeyChangeAmount(val As Long)
Attribute KeyChangeAmount.VB_Description = "Amount to change the value when the user presses arrow keys"
    value_change = val

    PropertyChanged "KeyChangeAmount"   ' Tell VB.
End Property

' *********************************************
' Set Enabled.
' *********************************************
Public Property Let Enabled(val As Boolean)
    UserControl.Enabled = val

    PropertyChanged "Enabled"   ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Compute the value we would have if (X, Y)
' were clicked.
' *********************************************
Private Function ComputeValue_Bar(the_canvas As PictureBox, X As Single, Y As Single) As Long
Dim fract As Single
Dim val As Long
Dim wid As Single
Dim hgt As Single

    ' No range. Pick the_min.
    If the_min >= the_max Then
        ComputeValue_Bar = the_min
        Exit Function
    End If
    
    ' See at what fraction of the way through
    ' the control the point lies.
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    
    Select Case the_orientation
        Case orientLeftToRight
            fract = X / wid
        Case orientRightToLeft
            fract = (wid - X) / wid
        Case orientBottomToTop
            fract = (hgt - Y) / hgt
        Case orientTopToBottom
            fract = Y / hgt
        Case Else
            ComputeValue_Bar = the_min
            Exit Function
    End Select

    ' Compute the result.
    val = the_min + _
        (the_max - the_min) * fract
    
    ' Keep the result in range.
    If val < the_min Then
        val = the_min
    ElseIf val > the_max Then
        val = the_max
    End If
    
    ComputeValue_Bar = val
End Function
' *********************************************
' Compute the value we would have if (X, Y)
' were clicked.
' *********************************************
Private Function ComputeValue(the_canvas As PictureBox, X As Single, Y As Single) As Long
    If Style = styleDial Then
        ComputeValue = ComputeValue_Dial(the_canvas, X, Y)
    Else
        ComputeValue = ComputeValue_Bar(the_canvas, X, Y)
    End If
End Function

' *********************************************
' Get the fill color.
' *********************************************
Public Property Get FillColor() As OLE_COLOR
Attribute FillColor.VB_Description = "Color of face for styleDial"
Attribute FillColor.VB_UserMemId = -510
    FillColor = UserControl.FillColor
End Property
' *********************************************
' Set the fill color.
' *********************************************
Public Property Let FillColor(clr As OLE_COLOR)
    UserControl.FillColor = clr
    Picture2D.FillColor = clr
    Picture3D.FillColor = clr

    PropertyChanged "FillColor" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Get the spacing between tic marks.
' *********************************************
Public Property Get TicSpacing() As Long
Attribute TicSpacing.VB_Description = "Distance in user units between tic marks for styleDial"
    TicSpacing = tic_spacing
End Property


' *********************************************
' Set the spacing between tic marks.
' *********************************************
Public Property Let TicSpacing(val As Long)
    tic_spacing = val

    PropertyChanged "TicSpacing"   ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property

' *********************************************
' Start selecting a value.
' *********************************************
Private Sub MouseDown(X As Single, Y As Single)
    ' Do nothing if there is no range or the
    ' control is disabled.
    If (the_min >= the_max) Or (Not Enabled) Then _
        Exit Sub
    
    ' Start dragging.
    dragging = True
    
    ' Update the value.
    Value = ComputeValue(the_canvas, X, Y)
    
    ' Generate a Scroll event.
    RaiseEvent Scroll
End Sub

' *********************************************
' Set the orientation.
' *********************************************
Public Property Let Orientation(val As GaugeOrientation)
Attribute Orientation.VB_Description = "Orientation and direction (left-to-right, etc.)"
    the_orientation = val

    PropertyChanged "Orientation" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Set the style.
' *********************************************
Public Property Let Style(val As GaugeStyle)
Attribute Style.VB_Description = "Style of gauge (styleBar, stylePic, etc.)"
    the_style = val

    PropertyChanged "Style" ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Get the orientation.
' *********************************************
Public Property Get Orientation() As GaugeOrientation
    Orientation = the_orientation
End Property
' *********************************************
' Get the style.
' *********************************************
Public Property Get Style() As GaugeStyle
    Style = the_style
End Property

' *********************************************
' Get the maximum value.
' *********************************************
Public Property Get Max() As Long
    Max = the_max
End Property
' *********************************************
' Get the Appearance.
' *********************************************
Public Property Get Appearance() As AppearanceValue
Attribute Appearance.VB_Description = "Determines 2D or 3D appearance"
Attribute Appearance.VB_UserMemId = -520
    Appearance = the_appearance
End Property

' *********************************************
' Get the amount to change the value when the
' user presses an arrow key.
' *********************************************
Public Property Get KeyChangeAmount() As Long
    KeyChangeAmount = value_change
End Property

' *********************************************
' Set the value.
' *********************************************
Public Property Let Value(val As Long)
Attribute Value.VB_Description = "The value selected"
    'Make sure the value is in bounds.
    If val < the_min Then
        val = the_min
    ElseIf val > the_max Then
        val = the_max
    End If

    ' Do nothing if the value has not changed.
    If the_value = val Then Exit Property
    
    ' Save the new value.
    the_value = val

    PropertyChanged "Value"   ' Tell VB.
    Paint   ' Redisplay the gauge.
End Property
' *********************************************
' Return the value.
' *********************************************
Public Property Get Value() As Long
    Value = the_value
End Property

' *********************************************
' The control now has the focus.
' *********************************************
Private Sub Picture2D_GotFocus()
    has_focus = True

    Paint   ' Redisplay the gauge.
End Sub

' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Picture2D_Paint()
    Paint
End Sub


' *********************************************
' The control now has the focus.
' *********************************************
Private Sub Picture3D_GotFocus()
    has_focus = True

    Paint   ' Redisplay the gauge.
End Sub

' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Picture3D_Paint()
    Paint
End Sub

' *********************************************
' Display the picture loaded asynchronously
' from a URL.
' *********************************************
Private Sub UserControl_AsyncReadComplete(AsyncProp As AsyncProperty)
    On Error GoTo AsyncReadError
    
    If AsyncProp.PropertyName = "FillPictureURL" Then
        Set FillPicture = AsyncProp.Value
    End If

AsyncReadError:

End Sub

' *********************************************
' Set default property values.
' *********************************************
Private Sub UserControl_InitProperties()
    loading = True
    
    Appearance = dflt_Appearance
    BackColor = Ambient.BackColor
    ForeColor = Ambient.ForeColor
    FillColor = dflt_FillColor
    Max = dflt_Max
    Min = dflt_Min
    Orientation = dflt_Orientation
    Style = dflt_Style
    Value = dflt_Value
    KeyChangeAmount = dflt_KeyChangeAmount
    TicSpacing = dflt_TicSpacing
    Set FillPicture = Nothing
    TicWidth = dflt_TicWidth
    NarrowColor = dflt_NarrowColor
    NarrowPercent = dflt_NarrowPercent
    FillPictureURL = dflt_FillPictureURL
    
    Enabled = dflt_Enabled

    ' Redraw now.
    loading = False
    Paint
End Sub

' *********************************************
' Respond to the arrow keys.
' *********************************************
Private Sub KeyDown(KeyCode As Integer)
Dim dv As Long
Dim val As Long

    ' See by how much the value should change.
    Select Case KeyCode
        Case vbKeyLeft
            If the_orientation = orientLeftToRight Then
                dv = -value_change
            ElseIf the_orientation = orientRightToLeft Then
                dv = value_change
            End If
        
        Case vbKeyRight
            If the_orientation = orientLeftToRight Then
                dv = value_change
            ElseIf the_orientation = orientRightToLeft Then
                dv = -value_change
            End If
        
        Case vbKeyUp
            If the_orientation = orientBottomToTop Then
                dv = value_change
            ElseIf the_orientation = orientTopToBottom Then
                dv = -value_change
            End If
        
        Case vbKeyDown
            If the_orientation = orientBottomToTop Then
                dv = -value_change
            ElseIf the_orientation = orientTopToBottom Then
                dv = value_change
            End If
        
        Case Else
            dv = 0
    End Select
    
    ' Make the change.
    If dv <> 0 Then
        ' Make sure the new value is in bounds.
        val = the_value + dv
        If val >= the_min And val <= the_max Then
            ' Update the value.
            Value = val
            
            ' Generate a Change event.
            RaiseEvent Change
        End If
    End If
End Sub
' *********************************************
' Respond to the arrow keys.
' *********************************************
Private Sub Picture2D_KeyDown(KeyCode As Integer, Shift As Integer)
    KeyDown KeyCode
End Sub
' *********************************************
' Respond to the arrow keys.
' *********************************************
Private Sub Picture3D_KeyDown(KeyCode As Integer, Shift As Integer)
    KeyDown KeyCode
End Sub

' *********************************************
' The control no longer has the focus.
' *********************************************
Private Sub Picture2D_LostFocus()
    has_focus = False
    
    Paint   ' Redisplay the gauge.
End Sub
' *********************************************
' The control no longer has the focus.
' *********************************************
Private Sub Picture3D_LostFocus()
    has_focus = False
    
    Paint   ' Redisplay the gauge.
End Sub

' *********************************************
' Start selecting a value.
' *********************************************
Private Sub Picture3d_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    MouseDown X, Y
End Sub
' *********************************************
' Start selecting a value.
' *********************************************
Private Sub Picture2D_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    MouseDown X, Y
End Sub
' *********************************************
' Continue selecting a value.
' *********************************************
Private Sub Picture3d_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    MouseMove X, Y
End Sub

' *********************************************
' Continue selecting a value.
' *********************************************
Private Sub Picture2D_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    MouseMove X, Y
End Sub
' *********************************************
' Continue selecting a value.
' *********************************************
Private Sub MouseMove(X As Single, Y As Single)
    ' Do nothing if the user is not selecting
    ' a value.
    If Not dragging Then Exit Sub

    ' Update the value.
    Value = ComputeValue(the_canvas, X, Y)
    
    ' Generate a Scroll event.
    RaiseEvent Scroll
End Sub

' *********************************************
' Set the new value.
' *********************************************
Private Sub Picture3d_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    MouseUp X, Y
End Sub

' *********************************************
' Set the new value.
' *********************************************
Private Sub Picture2D_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    MouseUp X, Y
End Sub
' *********************************************
' Set the new value.
' *********************************************
Private Sub MouseUp(X As Single, Y As Single)
    ' Do nothing if the user is not selecting
    ' a value.
    If Not dragging Then Exit Sub
    dragging = False
    
    ' Update the value.
    Value = ComputeValue(the_canvas, X, Y)
    
    ' Generate a Change event.
    RaiseEvent Change
End Sub

' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Paint_Bar()
Dim v As Single
Dim wid As Single
Dim hgt As Single

    ' Do nothing if we're loading.
    If loading Then Exit Sub
    
    ' Start from scratch.
    the_canvas.Cls
                        
    ' See how much of the gauge should be filled.
    If the_min >= the_max Then
        ' Fill in nothing.
        v = 0#
    Else
        v = (the_value - the_min) / (the_max - the_min)
    End If
        
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    
    ' Convert the value to pixels.
    If the_orientation = orientLeftToRight Or _
       the_orientation = orientRightToLeft _
    Then
        v = v * wid
    Else
        v = v * hgt
    End If
    
    ' Draw the gauge with the correct orientation.
    Select Case the_orientation
        Case orientLeftToRight
            the_canvas.Line (0, 0)-(v, hgt), , BF
        Case orientRightToLeft
            the_canvas.Line (wid, 0)-(wid - v, hgt), , BF
        Case orientBottomToTop
            the_canvas.Line (0, hgt)-(wid, hgt - v), , BF
        Case orientTopToBottom
            the_canvas.Line (0, 0)-(wid, v), , BF
    End Select
    
    ' If the gauge has the focus, outline it.
    If has_focus Then
        the_canvas.DrawStyle = vbDot
        the_canvas.DrawMode = vbInvert
        the_canvas.Line (0, 0)- _
            (wid - 1, hgt - 1), _
            , B
        the_canvas.DrawStyle = vbSolid
        the_canvas.DrawMode = vbCopyPen
    End If
End Sub
' *********************************************
' Draw the gauge.
' *********************************************
Private Sub Paint()
    Select Case Style
        Case styleBar
            Paint_Bar
        Case styleDial
            Paint_Dial
        Case stylePic
            Paint_Pic
        Case styleTic
            Paint_Tic
        Case styleWid
            Paint_Wid
    End Select
End Sub

' *********************************************
' Read the property values.
' *********************************************
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    ' Do not redraw.
    loading = True
    
    Appearance = PropBag.ReadProperty("Appearance", dflt_Appearance)
    BackColor = PropBag.ReadProperty("BackColor", Ambient.BackColor)
    ForeColor = PropBag.ReadProperty("ForeColor", Ambient.ForeColor)
    FillColor = PropBag.ReadProperty("FillColor", dflt_FillColor)
    Max = PropBag.ReadProperty("Max", dflt_Max)
    Min = PropBag.ReadProperty("Min", dflt_Min)
    Orientation = PropBag.ReadProperty("Orientation", dflt_Orientation)
    Style = PropBag.ReadProperty("Style", dflt_Style)
    Value = PropBag.ReadProperty("Value", dflt_Value)
    KeyChangeAmount = PropBag.ReadProperty("KeyChangeAmount", dflt_KeyChangeAmount)
    TicSpacing = PropBag.ReadProperty("TicSpacing", dflt_TicSpacing)
    Set FillPicture = PropBag.ReadProperty("FillPicture", Nothing)
    TicWidth = PropBag.ReadProperty("TicWidth", dflt_TicWidth)
    NarrowColor = PropBag.ReadProperty("NarrowColor", dflt_NarrowColor)
    NarrowPercent = PropBag.ReadProperty("NarrowPercent", dflt_NarrowPercent)
    FillPictureURL = PropBag.ReadProperty("FillPictureURL", dflt_FillPictureURL)

    Enabled = PropBag.ReadProperty("Enabled", dflt_Enabled)

    ' Redraw now.
    loading = False
    Paint
End Sub

' *********************************************
' Make the picture boxes fill the drawing area.
' *********************************************
Private Sub UserControl_Resize()
    Picture2D.Move 0, 0, ScaleWidth, ScaleHeight
    Picture3D.Move 0, 0, ScaleWidth, ScaleHeight
End Sub

' *********************************************
' Save the property values.
' *********************************************
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    PropBag.WriteProperty "Appearance", Appearance, dflt_Appearance
    PropBag.WriteProperty "BackColor", BackColor, Ambient.BackColor
    PropBag.WriteProperty "ForeColor", ForeColor, Ambient.ForeColor
    PropBag.WriteProperty "FillColor", FillColor, dflt_FillColor
    PropBag.WriteProperty "Max", Max, dflt_Max
    PropBag.WriteProperty "Min", Min, dflt_Min
    PropBag.WriteProperty "Orientation", Orientation, dflt_Orientation
    PropBag.WriteProperty "Style", Style, dflt_Style
    PropBag.WriteProperty "Value", Value, dflt_Value
    PropBag.WriteProperty "KeyChangeAmount", KeyChangeAmount, dflt_KeyChangeAmount
    PropBag.WriteProperty "TicSpacing", TicSpacing, dflt_TicSpacing
    PropBag.WriteProperty "FillPicture", FillPicture, Nothing
    PropBag.WriteProperty "TicWidth", TicWidth, dflt_TicWidth
    PropBag.WriteProperty "NarrowColor", NarrowColor, dflt_NarrowColor
    PropBag.WriteProperty "NarrowPercent", NarrowPercent, dflt_NarrowPercent
    PropBag.WriteProperty "FillPictureURL", FillPictureURL, dflt_FillPictureURL

    PropBag.WriteProperty "Enabled", Enabled, dflt_Enabled
End Sub
' *********************************************
' Compute the value we would have if (X, Y)
' were clicked.
' *********************************************
Private Function ComputeValue_Dial(the_canvas As PictureBox, X As Single, Y As Single) As Long
Dim fract As Single
Dim val As Long
Dim wid As Single
Dim hgt As Single
Dim x0 As Single
Dim y0 As Single
Dim dx As Single
Dim dy As Single
Dim theta As Single

    ' No range. Pick the_min.
    If the_min >= the_max Then
        ComputeValue_Dial = the_min
        Exit Function
    End If
    
    ' See at what fraction of the way through
    ' the range the point lies.
    wid = the_canvas.ScaleWidth
    hgt = the_canvas.ScaleHeight
    x0 = wid / 2
    y0 = hgt - 1
    dx = X - x0
    dy = y0 - Y
    theta = Arctan2(dx, dy)

    ' Convert that into Dial units.
    Select Case the_orientation
        Case orientLeftToRight
            fract = (PI - theta) / PI
        
        Case orientRightToLeft
            fract = theta / PI
        
        Case Else
            ComputeValue_Dial = the_min
            Exit Function
    End Select

    ' Compute the result.
    val = the_min + _
        (the_max - the_min) * fract
    
    ' Keep the result in range.
    If val < the_min Then
        val = the_min
    ElseIf val > the_max Then
        val = the_max
    End If
    
    ComputeValue_Dial = val
End Function


